-- Name: A2A Engagements
-- Author: Delta99
-- Date Created: 22 May 2017

-- Define some debug variables

D99_DebugRedSpawnPlanesDisable = D99_DebugRedSpawnPlanesDisable or false
D99_DebugRedSpawnGroundDisable = D99_DebugRedSpawnGroundDisable or false
D99_DebugTrace = D99_DebugTrace or false

function D99DebugTrace(output)
  if D99_DebugTrace then
    BASE:E(output)
  end
end 

-- Global Arrays:

MenuBlue = {}
groupToSpawn = {}
groupsForSide = {}

groupsForSide[coalition.side.RED] = coalition.getGroups(coalition.side.RED)
groupsForSide[coalition.side.BLUE] = coalition.getGroups(coalition.side.BLUE)

-- Global Variables

altitude = {}
altitude["Min"] = {}
altitude["Max"] = {}
altitude["Min"]["GLOBAL"] = 100
altitude["Min"][coalition.side.RED] = 500        -- Altitudes in feet
altitude["Max"][coalition.side.RED] = 10000

spawnDistance = {}
spawnDistance["Min"] = {}
spawnDistance["Max"] = {}
spawnDistance["Min"]["GLOBAL"] = 5
spawnDistance["Min"][coalition.side.RED] = 20    -- distance to spawn in NM
spawnDistance["Max"][coalition.side.RED] = 60

spawnUnits = {}
spawnUnits[coalition.side.RED] = 1
spawnUnits[coalition.side.BLUE] = 1

-- Have a scheduler run every so often to build the group menu's

SCHEDULER:New( nil,
  function()
    -- Got this code from Functional.Detection.lua  
    _DATABASE:ForEachPlayer(
      --- @param Wrapper.Unit#UNIT firstPlayerUnit
      function( PlayerUnitName )
        local firstPlayerUnitFound = UNIT:FindByName( PlayerUnitName )
        local menuBuilt = firstPlayerUnitFound:GetState(firstPlayerUnitFound, "MenuBuilt") or false
        
        if (not menuBuilt) then
          firstPlayerUnitFound:SetState(firstPlayerUnitFound, "MenuBuilt", true)
          buildGroupMenu(firstPlayerUnitFound:GetGroup())
        end
      end
    )
  end, {}, 10, 30 )

-- @param Wrapper.Group#GROUP groupSpawned The group spawned 
function setREDRoute(groupSpawned, playerPos, randPos)
  D99DebugTrace(groupSpawned)
  
  local coordCurr = COORDINATE:NewFromVec3(randPos)
  local coordDest = COORDINATE:NewFromVec3(playerPos)

  D99DebugTrace(coordCurr)
  D99DebugTrace(coordDest)
    
  local headingToDest = coordCurr:HeadingTo(coordDest)
  local headingDistance = coordCurr:Get2DDistance(coordDest)

  D99DebugTrace(headingToDest)
  D99DebugTrace(headingDistance)

  local distanceOneThird = headingDistance / 3
  local newAngle = getNewAngle(headingToDest)

  D99DebugTrace(distanceOneThird)
  D99DebugTrace(newAngle)
  
  local coordWaypoints = {}
  
  coordWaypoints[1] = coordCurr:Translate(distanceOneThird, newAngle, true)
  
  headingToDest = coordWaypoints[1]:HeadingTo(coordDest)
  newAngle = getNewAngle(headingToDest)
  
  coordWaypoints[2] = coordWaypoints[1]:Translate(distanceOneThird, newAngle, true)

  headingToDest = coordWaypoints[2]:HeadingTo(coordDest)
  newAngle = getNewAngle(headingToDest)
  
  coordWaypoints[3] = coordWaypoints[2]:Translate(distanceOneThird, newAngle, true)
  
  D99DebugTrace(coordWaypoints[1])
  D99DebugTrace(coordWaypoints[2])
  D99DebugTrace(coordWaypoints[3])

  local coordRoute = {}
  
  for i=1, 3 do
    coordRoute[i] = coordWaypoints[i]:WaypointAirTurningPoint(AI.Task.AltitudeType.RADIO, UTILS.KnotsToMps(math.random(550, 700)))
  
    if (D99_DebugTrace) then
      coordWaypoints[i]:MarkToAll("Waypoint " .. i .. " Alt: " .. coordWaypoints[i]:GetAltitudeText() .. " Vel: " .. coordWaypoints[i]:GetMovingText())
    end
  end
  
  local enrouteTask -- Dcs.DCSTask#Task
  
  enrouteTask = groupSpawned:EnRouteTaskEngageTargets(UTILS.NMToMeters(120), {"Battle airplanes"}, 1)
  D99DebugTrace(enrouteTask)
  
  groupSpawned:Route(coordRoute)
  
  groupSpawned:PushTask(enrouteTask)
  groupSpawned:OptionROEWeaponFree()
end

function getNewAngle(heading)
  local newAngle = heading + (math.random(1, 90) - 45)
  
  if (newAngle < 0) then
    newAngle = 360 + newAngle
  end
  
  return newAngle
end

function spawnRandom(sideToSpawn, group)
  if (group ~= nil) then

    D99DebugTrace(groupsForSide[sideToSpawn])
  
    -- TODO: Might want to check for if we pass something other than RED or BLUE (abort function?)  
    local templatePrefix = "BLUE"
    local groupNameCheckSize = 7
    if (sideToSpawn == coalition.side.RED) then
      templatePrefix = "RED"
      groupNameCheckSize = 6
    end
    
    local templateRandom = {}
    for DCSGroupId, DCSGroup in pairs( groupsForSide[sideToSpawn] ) do
      if DCSGroup:isExist() then
        local DCSGroupName = DCSGroup:getName()
        D99DebugTrace(DCSGroupName)
        
        if (string.sub(DCSGroupName, 1, groupNameCheckSize) == templatePrefix .. " AI") then
          templateRandom[#templateRandom+1] = DCSGroupName
        end
      end
    end
    
    D99DebugTrace(templateRandom)
    
    local spawnIndex = #groupToSpawn+1 
    local tempRandGroup = math.random(1,table.getn(templateRandom))
    local groupAlias = templateRandom[tempRandGroup] .. "-" .. spawnIndex
    D99DebugTrace("About to spawn alias: " .. groupAlias)
    
    local playerPos = POINT_VEC3:NewFromVec3(group:GetVec3())
    local randHeight = math.random(altitude["Min"][sideToSpawn], altitude["Max"][sideToSpawn])
    D99DebugTrace("randHeight: " .. randHeight)
    local randPosVec2 = playerPos:GetRandomVec2InRadius(UTILS.NMToMeters(spawnDistance["Max"][sideToSpawn]), UTILS.NMToMeters(spawnDistance["Min"][sideToSpawn]))
    local randPos = POINT_VEC3:NewFromVec2(randPosVec2)
    
    -- NOTE: The randHeight in previous call here doesn't seem to be working so lets use SetY instead
    -- FIXME: Not sure this is really setting the altitude properly
    local randPosVec2Height = land.getHeight(randPosVec2)
    D99DebugTrace(UTILS.MetersToFeet(randPosVec2Height))
    randPos:SetY(randPosVec2Height + UTILS.FeetToMeters(randHeight))
    D99DebugTrace(land.getHeight(randPos))
    D99DebugTrace(randPos)
    
    -- newAliasSpawn = SPAWN:NewWithAlias(templateRandom[tempRandGroup], groupAlias):OnSpawnGroup(setREDRoute, playerPos, randPos)
    groupToSpawn[spawnIndex] = SPAWN:NewWithAlias(templateRandom[tempRandGroup], groupAlias):OnSpawnGroup(setREDRoute, playerPos, randPos):SpawnFromVec3(randPos)
    
    -- Lets see if we are spawning more than 1 unit
    if (spawnUnits[sideToSpawn] > 1) then
      for i=2, spawnUnits[sideToSpawn] do
        local additionalGroupAlias = groupAlias .. "-" .. i
        D99DebugTrace("About to spawn additional alias: " .. additionalGroupAlias)
        local additionalSpawnPos = randPos
        additionalSpawnPos:AddX(100 * (i-1))
        additionalSpawnPos:AddZ(100 * (i-1)) 
        local additionalSpawn = SPAWN:NewWithAlias(templateRandom[tempRandGroup], additionalGroupAlias):SpawnFromVec3(additionalSpawnPos)
        local escortVec3 = POINT_VEC3:New(UTILS.NMToMeters(-1 * (i-1)), 0, UTILS.NMToMeters(1 * (i-1)))
        local escortTask = additionalSpawn:TaskEscort(groupToSpawn[spawnIndex], escortVec3, nil, UTILS.NMToMeters(30), {"Air"})
        additionalSpawn:PushTask(escortTask, 5)
      end
    end
  else
    local msgText = "No player found, cannot spawn at the moment. Try again in a few minutes" 
    MESSAGE:New(msgText, 20):ToAll()
  end
end

function optionsAltitudeChange(minMax, side, value)
  D99DebugTrace(altitude[minMax][side])
  altitude[minMax][side] = altitude[minMax][side] + value
  D99DebugTrace(altitude[minMax][side])

  -- make sure min value is not below the global min value. For max we don't care.  
  if (altitude["Min"][side] < altitude["Min"]["GLOBAL"]) then
    altitude["Min"][side] = altitude["Min"]["GLOBAL"]
    D99DebugTrace(altitude[minMax][side])
  end
  
  -- make sure min and max are not equal or in the opposite direction of each other. If so, make max larger than min
  if (altitude["Max"][side] <= altitude["Min"][side]) then
    altitude["Max"][side] = altitude["Min"][side] + altitude["Min"]["GLOBAL"]
  end
  
  if (altitude["Min"][side] >= altitude["Max"][side]) then
    altitude["Max"][side] = altitude["Min"][side] + altitude["Min"]["GLOBAL"]
  end
  
  local msgText = "Side: " .. side .. " Altitude Min: " .. altitude["Min"][side] .. ", Max: " .. altitude["Max"][side] 
  MESSAGE:New(msgText, 20):ToAll()
end

function optionsDistChange(minMax, side, value)
  D99DebugTrace(spawnDistance[minMax][side])
  spawnDistance[minMax][side] = spawnDistance[minMax][side] + value
  D99DebugTrace(spawnDistance[minMax][side])

  -- make sure min value is not below the global min value. For max we don't care.  
  if (spawnDistance["Min"][side] < spawnDistance["Min"]["GLOBAL"]) then
    spawnDistance["Min"][side] = spawnDistance["Min"]["GLOBAL"]
    D99DebugTrace(spawnDistance[minMax][side])
  end
  
  -- make sure min and max are not equal or in the opposite direction of each other. If so, make max larger than min
  if (spawnDistance["Max"][side] <= spawnDistance["Min"][side]) then
    spawnDistance["Max"][side] = spawnDistance["Min"][side] + spawnDistance["Min"]["GLOBAL"]
  end
  
  if (spawnDistance["Min"][side] >= spawnDistance["Max"][side]) then
    spawnDistance["Max"][side] = spawnDistance["Min"][side] + spawnDistance["Min"]["GLOBAL"]
  end

  local msgText = "Side: " .. side .. " Distance Min: " .. spawnDistance["Min"][side] .. ", Max: " .. spawnDistance["Max"][side] 
  MESSAGE:New(msgText, 20):ToAll()
end

function optionsNumChange(side, value)
  D99DebugTrace(spawnUnits[side])
  spawnUnits[side] = spawnUnits[side] + value
  
  -- make sure the value is not less than 1
  if (spawnUnits[side] < 1) then
    spawnUnits[side] = 1
  end
  D99DebugTrace(spawnUnits[side])   

  local msgText = "Side: " .. side .. " Spawn Units: " .. spawnUnits[side] 
  MESSAGE:New(msgText, 20):ToAll()
end

function buildGroupMenu(group)
  local MenuBlueAltOptions = MENU_GROUP:New(group, "Spawn Alt Options")
  local MenuBlueDistOptions = MENU_GROUP:New(group, "Spawn Distance Options")
  local MenuBlueNumOptions = MENU_GROUP:New(group, "Spawn Number Options")
  local MenuBlueSpawn = MENU_GROUP:New(group, "Spawn Flight")
  
  -- Altitude Option Menu Items
  local MenuBlueOptionsAltMinIncrease = MENU_GROUP:New(group, "Min Increase", MenuBlueAltOptions)
  local MenuBlueOptionsAltMinIncrease100 = MENU_GROUP_COMMAND:New(group, "Inc by 100 ft", MenuBlueOptionsAltMinIncrease, optionsAltitudeChange, "Min", coalition.side.RED, 100)
  local MenuBlueOptionsAltMinIncrease500 = MENU_GROUP_COMMAND:New(group, "Inc by 500 ft", MenuBlueOptionsAltMinIncrease, optionsAltitudeChange, "Min", coalition.side.RED, 500)
  local MenuBlueOptionsAltMinIncrease1000 = MENU_GROUP_COMMAND:New(group, "Inc by 1,000 ft", MenuBlueOptionsAltMinIncrease, optionsAltitudeChange, "Min", coalition.side.RED, 1000)
  local MenuBlueOptionsAltMinIncrease5000 = MENU_GROUP_COMMAND:New(group, "Inc by 5,000 ft", MenuBlueOptionsAltMinIncrease, optionsAltitudeChange, "Min", coalition.side.RED, 5000)

  local MenuBlueOptionsAltMinDecrease = MENU_GROUP:New(group, "Min Decrease", MenuBlueAltOptions)
  local MenuBlueOptionsAltMinDecrease100 = MENU_GROUP_COMMAND:New(group, "Dec by 100 ft", MenuBlueOptionsAltMinDecrease, optionsAltitudeChange, "Min", coalition.side.RED, -100)
  local MenuBlueOptionsAltMinDecrease500 = MENU_GROUP_COMMAND:New(group, "Dec by 100 ft", MenuBlueOptionsAltMinDecrease, optionsAltitudeChange, "Min", coalition.side.RED, -500)
  local MenuBlueOptionsAltMinDecrease1000 = MENU_GROUP_COMMAND:New(group, "Dec by 1,000 ft", MenuBlueOptionsAltMinDecrease, optionsAltitudeChange, "Min", coalition.side.RED, -1000)
  local MenuBlueOptionsAltMinDecrease5000 = MENU_GROUP_COMMAND:New(group, "Dec by 5,000 ft", MenuBlueOptionsAltMinDecrease, optionsAltitudeChange, "Min", coalition.side.RED, -5000)

  local MenuBlueOptionsAltMaxIncrease = MENU_GROUP:New(group, "Max Increase", MenuBlueAltOptions)
  local MenuBlueOptionsAltMaxIncrease100 = MENU_GROUP_COMMAND:New(group, "Inc by 100 ft", MenuBlueOptionsAltMaxIncrease, optionsAltitudeChange, "Max", coalition.side.RED, 100)
  local MenuBlueOptionsAltMaxIncrease500 = MENU_GROUP_COMMAND:New(group, "Inc by 500 ft", MenuBlueOptionsAltMaxIncrease, optionsAltitudeChange, "Max", coalition.side.RED, 500)
  local MenuBlueOptionsAltMaxIncrease1000 = MENU_GROUP_COMMAND:New(group, "Inc by 1,000 ft", MenuBlueOptionsAltMaxIncrease, optionsAltitudeChange, "Max", coalition.side.RED, 1000)
  local MenuBlueOptionsAltMaxIncrease5000 = MENU_GROUP_COMMAND:New(group, "Inc by 5,000 ft", MenuBlueOptionsAltMaxIncrease, optionsAltitudeChange, "Max", coalition.side.RED, 5000)

  local MenuBlueOptionsAltMaxDecrease = MENU_GROUP:New(group, "Max Decrease", MenuBlueAltOptions)
  local MenuBlueOptionsAltMaxDecrease100 = MENU_GROUP_COMMAND:New(group, "Dec by 100 ft", MenuBlueOptionsAltMaxDecrease, optionsAltitudeChange, "Max", coalition.side.RED, -100)
  local MenuBlueOptionsAltMaxDecrease500 = MENU_GROUP_COMMAND:New(group, "Dec by 100 ft", MenuBlueOptionsAltMaxDecrease, optionsAltitudeChange, "Max", coalition.side.RED, -500)
  local MenuBlueOptionsAltMaxDecrease1000 = MENU_GROUP_COMMAND:New(group, "Dec by 1,000 ft", MenuBlueOptionsAltMaxDecrease, optionsAltitudeChange, "Max", coalition.side.RED, -1000)
  local MenuBlueOptionsAltMaxDecrease5000 = MENU_GROUP_COMMAND:New(group, "Dec by 5,000 ft", MenuBlueOptionsAltMaxDecrease, optionsAltitudeChange, "Max", coalition.side.RED, -5000)
  
  -- Distance Option Menu Items
  local MenuBlueOptionsDistMinIncrease = MENU_GROUP:New(group, "Min Increase", MenuBlueDistOptions)
  local MenuBlueOptionsDistMinIncrease5 = MENU_GROUP_COMMAND:New(group, "Inc by 5nm", MenuBlueOptionsDistMinIncrease, optionsDistChange, "Min", coalition.side.RED, 5)
  local MenuBlueOptionsDistMinIncrease10 = MENU_GROUP_COMMAND:New(group, "Inc by 10nm", MenuBlueOptionsDistMinIncrease, optionsDistChange, "Min", coalition.side.RED, 10)
  local MenuBlueOptionsDistMinIncrease25 = MENU_GROUP_COMMAND:New(group, "Inc by 25nm", MenuBlueOptionsDistMinIncrease, optionsDistChange, "Min", coalition.side.RED, 25)
  
  local MenuBlueOptionsDistMinDecrease = MENU_GROUP:New(group, "Min Decrease", MenuBlueDistOptions)
  local MenuBlueOptionsDistMinDecrease5 = MENU_GROUP_COMMAND:New(group, "Dec by 5nm", MenuBlueOptionsDistMinDecrease, optionsDistChange, "Min", coalition.side.RED, -5)
  local MenuBlueOptionsDistMinDecrease10 = MENU_GROUP_COMMAND:New(group, "Dec by 10nm", MenuBlueOptionsDistMinDecrease, optionsDistChange, "Min", coalition.side.RED, -10)
  local MenuBlueOptionsDistMinDecrease25 = MENU_GROUP_COMMAND:New(group, "Dec by 25nm", MenuBlueOptionsDistMinDecrease, optionsDistChange, "Min", coalition.side.RED, -25)

  local MenuBlueOptionsDistMaxIncrease = MENU_GROUP:New(group, "Max Increase", MenuBlueDistOptions)
  local MenuBlueOptionsDistMaxIncrease5 = MENU_GROUP_COMMAND:New(group, "Inc by 5nm", MenuBlueOptionsDistMaxIncrease, optionsDistChange, "Max", coalition.side.RED, 5)
  local MenuBlueOptionsDistMaxIncrease10 = MENU_GROUP_COMMAND:New(group, "Inc by 10nm", MenuBlueOptionsDistMaxIncrease, optionsDistChange, "Max", coalition.side.RED, 10)
  local MenuBlueOptionsDistMaxIncrease25 = MENU_GROUP_COMMAND:New(group, "Inc by 25nm", MenuBlueOptionsDistMaxIncrease, optionsDistChange, "Max", coalition.side.RED, 25)
  
  local MenuBlueOptionsDistMaxDecrease = MENU_GROUP:New(group, "Max Decrease", MenuBlueDistOptions)
  local MenuBlueOptionsDistMaxDecrease5 = MENU_GROUP_COMMAND:New(group, "Dec by 5nm", MenuBlueOptionsDistMaxDecrease, optionsDistChange, "Max", coalition.side.RED, -5)
  local MenuBlueOptionsDistMaxDecrease10 = MENU_GROUP_COMMAND:New(group, "Dec by 10nm", MenuBlueOptionsDistMaxDecrease, optionsDistChange, "Max", coalition.side.RED, -10)
  local MenuBlueOptionsDistMaxDecrease25 = MENU_GROUP_COMMAND:New(group, "Dec by 25nm", MenuBlueOptionsDistMaxDecrease, optionsDistChange, "Max", coalition.side.RED, -25)

  -- Number Option Menu Items
  local MenuBlueOptionsNumAdd = MENU_GROUP_COMMAND:New(group, "Increase by 1", MenuBlueNumOptions, optionsNumChange, coalition.side.RED, 1)
  local MenuBlueOptionsNumDec = MENU_GROUP_COMMAND:New(group, "Decrease by 1", MenuBlueNumOptions, optionsNumChange, coalition.side.RED, -1)

  -- Spawn Menu items
  local MenuBlueSpawnRandom = MENU_GROUP_COMMAND:New(group, "Random", MenuBlueSpawn, spawnRandom, coalition.side.RED, group)
  
  MESSAGE:New("Menus built"):ToGroup(group)
end
